import {VRoomEngine} from "./VRoom.js";
import PrimitiveRenderer from "./renderers/PrimitiveRenderer.js";

class DebugLine extends PrimitiveRenderer {
    constructor(from, to, color, width=1) {
        super({colors: color});
        this.vertices = [...from.glVec3, ...to.glVec3];
        this.indices = [0, 1];
        this.width = width;
    }

    _drawElements(gl) {
        // draw line
        gl.lineWidth(this.width);
        gl.drawElements(gl.LINES, this.indices.length, gl.UNSIGNED_SHORT, 0);
        gl.lineWidth(1);
    }
}


class Debug {
    constructor() {
        this.debugObjects = [];
        this.gl = null;
    }

    init(gl) {
        this.gl = gl;
    }

    update() {
        this.debugObjects = [];
    }

    render(renderInfo) {
        this.debugObjects.forEach((obj) => obj.render(renderInfo));
    }

    drawLine(from, to, color) {
        const line = new DebugLine(from, to, color);
        line.init(this.gl);
        this.debugObjects.push(line);
    }
}

export default Debug = new Debug();