import { VRoomEngine } from "./VRoom.js";
import PrimitiveRenderer from "./renderers/PrimitiveRenderer.js";
import Ray from "./Ray.js";
import { getPoseMatrix } from "./util/MatrixUtil.js";

class Physics {
    constructor() {

    }

    /**
     * Perform raycast and returns the hit game object or null
     * 
     * @param {*} origin 
     * @param {*} direction 
     * @param {*} layer 
     */
    raycast(origin, direction, layer=-1) {
        const ray = new Ray(origin, direction);
        let hits = [];
        // iterate over all game objects in the current scene and check if it has a PrimitiveRenderer component. If it has forward raycast
        for(let i = 0; i < VRoomEngine.currentScene.gameObjects.length; i++) {
            const go = VRoomEngine.currentScene.gameObjects[i];
            // check if on correct layer
            if (layer != -1 && layer != go.layer)
                continue;
            if (go.hasComponent(PrimitiveRenderer)) {
                if (go.getComponent(PrimitiveRenderer).raycast(ray)) {
                    // object is hit. lets add it to our list
                    hits.push(go);
                }
            }
        }
        // sort hits by distance to ray origin
        hits.sort((g1, g2) => {
            return g1.transform.position.sub(origin).length() - g2.transform.position.sub(origin).length();
        });

        if (hits.length > 0) 
            return hits[0];
        return null;
    }
}

export default Physics = new Physics();