export default class Quaternion {
    constructor(x=0, y=0, z=0, w=0) {
        this._quat = glMatrix.quat.fromValues(x, y, z, w);
    }

    get x() {
        return this._quat[0];
    }

    get y() {
        return this._quat[1];
    }

    get z() {
        return this._quat[2];
    }

    get w() {
        return this._quat[3];
    }

    get glQuat() {
        return glMatrix.quat.clone(this._quat);
    }

    clone() {
        return new Quaternion(this.x, this.y, this.z, this.w);
    }

    toEuler() {
        // roll (x-axis rotation)
        const sinr_cosp = +2.0 * (this.w * this.x + this.y * this.z);
        const cosr_cosp = +1.0 - 2.0 * (this.x * this.x + this.y * this.y);
        const roll = Math.atan2(sinr_cosp, cosr_cosp);

        // pitch (y-axis rotation)
        const sinp = +2.0 * (this.w * this.y - this.z * this.x);
        let pitch = 0;
        if (Math.abs(sinp) >= 1)
            pitch = (sinp > 0) ? (M_PI / 2) : (-M_PI / 2); // use 90 degrees if out of range
        else
            pitch = Math.asin(sinp);

        // yaw (z-axis rotation)
        const siny_cosp = +2.0 * (this.w * this.z + this.x * this.y);
        const cosy_cosp = +1.0 - 2.0 * (this.y * this.y + this.z * this.z);  
        const yaw = Math.atan2(siny_cosp, cosy_cosp);

        return [roll * 180 / Math.PI, pitch * 180 / Math.PI, yaw * 180 / Math.PI];
    }

    static fromEuler(roll, pitch, yaw) {
        const quat = new Quaternion();
        glMatrix.quat.fromEuler(quat._quat, roll, pitch, yaw);
        return quat;
    }
}