export default class Vector3 {
    constructor(x=0, y=0, z=0) {
        this._vec = glMatrix.vec3.fromValues(x, y, z);
    }

    get x() {
        return this._vec[0];
    }

    get y() {
        return this._vec[1];
    }

    get z() {
        return this._vec[2];
    }

    get glVec3() {
        return glMatrix.vec3.clone(this._vec);
    }

    toString() {
        return `Vector3(${this.x}, ${this.y}, ${this.z})`;
    }

    clone() {
        return new Vector3(this.x, this.y, this.z);
    }

    add(other) {
        return new Vector3(this.x + other.x, this.y + other.y, this.z + other.z);
    }

    subtract(other) {
        return new Vector3(this.x - other.x, this.y - other.y, this.z - other.z);
    }

    sub(other) {
        return this.subtract(other);
    }

    normalized() { 
        const normalized = new Vector3();
        glMatrix.vec3.normalize(normalized._vec, this._vec);
        return normalized;
    }

    dot(other) {
        return this.x * other.x + this.y * other.y + this.z * other.z;
    }

    cross(other) {
        return new Vector3(this.y * other.z - this.z * other.y,
                           this.z * other.x - this.x * other.z,
                           this.x * other.y - this.y * other.x);
    }

    multiply(scalar) {
        return new Vector3(this.x * scalar, this.y * scalar, this.z * scalar);
    }

    length() {
        return Math.sqrt(this.dot(this));
    }

    static get zero() {
        return new Vector3();
    }

    static get one() {
        return new Vector3(1, 1, 1);
    }

    static get forward() {
        return new Vector3(0, 0, -1);
    }

    static get up() {
        return new Vector3(0, 1, 0);
    }

    static get right() {
        return new Vector3(1, 0, 0);
    }
}