import PrimitiveRenderer from "./PrimitiveRenderer.js";
import {
    buildSphereVertices, buildSphereIndices
} from "../util/SphereUtil.js";

export default class HalfSphereRenderer extends PrimitiveRenderer {
    constructor(radius, colors, verticalDivisions = 100, horizontalDivisions = 20) {
        super({colors});
        this.initShape(radius, verticalDivisions, horizontalDivisions);
    }

    /**
     * Initializes the shape. We use the upper half of a UV sphere since it is the easiest to set up
     */
    initShape(radius, verticalDivisions, horizontalDivisions) {
        // we only want to deal with even number of horizontal divisions since it is easier for now
        if (horizontalDivisions % 2 != 0)
            horizontalDivisions++;
        // initialize the vertices for a UV sphere
        this.vertices = buildSphereVertices(radius, verticalDivisions, horizontalDivisions, 0, Math.ceil((horizontalDivisions + 1)/ 2));

        // number of vertices in half sphere (expect for top)
        const sphereVertexCount = this.vertices.length / 3 - 1;

        // set up the index array for the tris
        this.indices = buildSphereIndices(sphereVertexCount, verticalDivisions, horizontalDivisions, 0, Math.ceil((horizontalDivisions + 1) / 2));
    }
}