import Component from "../engine/Component.js";
import Input from "../engine/Input.js";
import Quaternion from "../engine/math/Quaternion.js";
import Vector3 from "../engine/math/Vector3.js";
import Physics from "../engine/Physics.js";
import Cube from "../game/Cube.js";
import CubeRenderer from "../engine/renderers/CubeRenderer.js";
import Debug from "../engine/Debug.js";
import GameObject from "../engine/GameObject.js";

export default class LaserPointer extends Component {
    constructor() {
        super();
        this.laserp = 0;
    }

    update(deltaTime) {
        // check if gamepad is connected
        if (Input.primaryGamepadConnected) {
            // update rotation from gamepad orientation
            this.transform.setRotationFromQuat(new Quaternion(...Input.primaryGamepad.pose.orientation));
            // check if trigger is pressed. only then do raycast
            if (Input.primaryGamepad.buttons[1].pressed) {
                // do raycast from laser position in laser direction
                const hit = Physics.raycast(this.transform.worldPosition, this.transform.forward, 1);
                // check if ray hit a cube
                if (hit && hit.hasComponent(CubeRenderer) && hit.hasComponent(Cube)) {
                    const cubeRenderer = hit.getComponent(CubeRenderer);
                    const cube = hit.getComponent(Cube);
                    cubeRenderer.setColors([1, 1, 0, 1]);
                    cube.goDie();
                }
            }
        }
    }
}